<?php
/**
 * @project    : yabandpay
 * @description:
 * @user       : PersiLiao
 * @email      : persi@sixsir.com
 * @date       : 2018/12/2
 * @time       : 22:57
 */

namespace YaBandPay\PersiLiao;

class Api
{
    const DOMAIN = 'https://mapi.yabandpay.com/';
    /**
     * @var Account
     */
    private $account;
    /**
     * @var Request
     */
    private $request;

    public function __construct(Account $account, Request $request)
    {
        $this->account = $account;
        $this->request = $request;
    }

    public function payment($pay_method, $order_id, $amount, $currency, $description, $redirect_url, $notify_url, $demo = '', $email='')
    {
        $params = [
            "user" => $this->account->getUser(),
            "method" => "v3.CreatePayments",
            "time" => \time(),
            "data" => [
                'pay_method' => 'online',
                "timeout" => "0",
                'sub_pay_method' => $pay_method,
                'order_id' => $order_id,
                'amount' => $amount,
                'currency' => $currency,
                'description' => $description,
                'redirect_url' => $redirect_url,
                'notify_url' => $notify_url,
                'demo' => $demo,
                "ideal_email" => $email,
                'post_email' =>  $email,
            ]
        ];

        return $this->request->post(self::paymentsUrl(), $params);
    }

    public function cancel($trade_id)
    {
        $params = [
            "user" => $this->account->getUser(),
            "method" => "v3.CancelPayOrder",
            "time" => \time(),
            "data" => [
                'trade_id' => $trade_id
            ]
        ];
        return $this->request->no_exception_post(self::paymentsUrl(), $params);
    }

    public function updateOrderId($trade_id,$order_id)
    {
        $params = [
            "user" => $this->account->getUser(),
            "method" => "v3.UpdateOrderId",
            "time" => \time(),
            "data" => [
                'trade_id' => $trade_id,
                'order_id' => $order_id
            ]
        ];
        return $this->request->no_exception_post(self::paymentsUrl(), $params);
    }

    private static function generateUrl($action)
    {
        return self::DOMAIN . $action;
    }

    protected static function paymentsUrl()
    {
        return self::generateUrl('Payments');
    }
}