<?php
/**
 * 2007-2019 PrestaShop
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 * @author    PrestaShop SA <contact@prestashop.com>
 * @copyright 2007-2019 PrestaShop SA
 * @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 *  International Registered Trademark & Property of PrestaShop SA
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

$vendor_autoload = \dirname(__FILE__) . '/vendor/autoload.php';

if (\file_exists($vendor_autoload) === false) {
    exit;
}

require_once $vendor_autoload;

use PrestaShop\PrestaShop\Core\Payment\PaymentOption;
use YaBandPay\PersiLiao\Account;
use YaBandPay\PersiLiao\Api;
use YaBandPay\PersiLiao\Cryptography;
use YaBandPay\PersiLiao\Payment;
use YaBandPay\PersiLiao\Request;

class Yabandpay extends PaymentModule
{
    const MODULE_NAME = 'yabandpay';
    
    public $display = 'view';
    
    public $module_key = '041c275731b7b4fb9730a9648c5dda25';
    
    protected $config_form = false;
    
    public $bootstrap = true;
    
    public $is_eu_compatible = 1;
    
    public $is_configurable = 1;
    
    protected $config
        = array(
            'YABANDPAY_ACCOUNT',
            'YABANDPAY_TOKEN',
            'YABANDPAY_EXCHANGERATE',
            'YABANDPAY_WECHATPAY',
            'YABANDPAY_ALIPAY',
            'YABANDPAY_IDEAL',
            'YABANDPAY_SOFORT',
            'YABANDPAY_BANCONTACT',
            'YABANDPAY_VISA',
            'YABANDPAY_MASTERCARD',
            'YABANDPAY_PAYPAL',
            // 'YABANDPAY_UNIONPAY',
            'YABANDPAY_UNIONPAYWAP',
            'YABANDPAY_ACTIVE',
            'YABANDPAY_EXCHANGE_RATE',
            'YABANDPAY_EXCHANGE_FEE',
            'YABANDPAY_EXCHANGE_CURRENCY',
            'YABANDPAY_DESC'
        );
    
    protected $pay_status
        = array(
            'YABANDPAY_OS_WAITING_WECHATPAY' => array(
                'zh' => '等待微信支付',
                'en' => 'Awaiting for WeChat Pay'
            ),
            'YABANDPAY_OS_WAITING_ALIPAY'    => array(
                'zh' => '等待支付宝支付',
                'en' => 'Awaiting for Alipay'
            ),
            'YABANDPAY_OS_WAITING_IDEAL' => array(
                'zh' => '等待iDeal支付',
                'en' => 'Awaiting for iDeal'
            ),
            'YABANDPAY_OS_WAITING_SOFORT' => array(
                'zh' => '等待Sofort支付',
                'en' => 'Awaiting for Sofort'
            ),
            'YABANDPAY_OS_WAITING_BANCONTACT' => array(
                'zh' => '等待Bancontact支付',
                'en' => 'Awaiting for Bancontact'
            ),
            'YABANDPAY_OS_WAITING_VISA' => array(
                'zh' => '等待Visa支付',
                'en' => 'Awaiting for Visa'
            ),
            'YABANDPAY_OS_WAITING_MASTERCARD' => array(
                'zh' => '等待Mastercard支付',
                'en' => 'Awaiting for Mastercard'
            ),
            'YABANDPAY_OS_WAITING_PAYPAL' => array(
                'zh' => '等待PayPal支付',
                'en' => 'Awaiting for PayPal'
            ),
            // 'YABANDPAY_OS_WAITING_UNIONPAY' => array(
            //     'zh' => '等待UnionPay支付',
            //     'en' => 'Awaiting for UnionPay'
            // ),
            'YABANDPAY_OS_WAITING_UNIONPAYWAP' => array(
                'zh' => '等待UnionPayWap支付',
                'en' => 'Awaiting for UnionPayWap'
            ),
        );
    
    private $tab_modules_list;
    
    public function __construct()
    {
        $this->name = 'yabandpay';
        $this->tab = 'payments_gateways';
        $this->version = '3.3.2';
        $this->author = 'YaBand';
        $this->need_instance = 1;
        $this->controllers = array('payment', 'notify');
        
        /**
         * Set $this->bootstrap to true if your module is compliant with bootstrap (PrestaShop 1.6)
         */
        $this->bootstrap = true;
        
        parent::__construct();
        
        $this->displayName = $this->l('YaBandPay');
        $this->description = $this->l(
            'To obtain the login credential, please visit our website www.yabandpay.com'
        );
        
        $this->confirmUninstall = $this->l('Are you sure to uninstall?');
        
        //$this->limited_countries = array('FR');
        
        $this->limited_currencies = array('EUR', 'CNY');
        
        $this->ps_versions_compliancy = array(
            'min' => '1.6',
            'max' => _PS_VERSION_
        );
        
        $this->tab_modules_list = _PS_ROOT_DIR_
            . ModuleCore::CACHE_FILE_TAB_MODULES_LIST;
    }
    
    /**
     * Don't forget to create update methods if needed:
     * http://doc.prestashop.com/display/PS16/Enabling+the+Auto-Update
     */
    public function install()
    {
        if (\extension_loaded('curl') == false) {
            $this->_errors[] = $this->l(
                'You have to enable the cURL extension on your server to install this module'
            );
            return false;
        }
        if (!parent::install()) {
            return false;
        }
        if (!$this->installXmlModules()) {
            $this->_errors[] = 'install module xml error';
            return false;
        }
        if (!$this->installOrderState()) {
            $this->_errors[] = 'install order state error';
            return false;
        }
        
        if (!$this->installSQL()) {
            $this->_errors[] = 'install sql error';
            return false;
        }
        
        if (!$this->registrationHook()) {
            $this->_errors[] = 'registration hook error';
            return false;
        }
        
        foreach ($this->config as $key) {
            if (!Configuration::updateValue($key, '')) {
                return false;
            }
        }
        
        Configuration::updateValue('YABANDPAY_LIVE_MODE', false);
        
        return true;
    }
    
    public function uninstall()
    {
        Configuration::deleteByName('YABANDPAY_LIVE_MODE');
        
        $this->config[] = 'YABANDPAY_OS_WAITING';
        $this->config = \array_merge(
            $this->config,
            array_keys(
                $this->pay_status
            )
        );
        foreach ($this->config as $key) {
            Configuration::deleteByName($key);
        }
        
        if (!$this->uninstallSQL()) {
            return false;
        }
        
        return parent::uninstall();
    }
    
    /**
     * Add the CSS & JavaScript files you want to be loaded in the BO.
     */
    public function hookBackOfficeHeader()
    {
        if (Tools::getValue('module_name') == $this->name) {
            $this->context->controller->addJS(
                $this->_path . 'views/js/back.js'
            );
            $this->context->controller->addCSS(
                $this->_path . 'views/css/back.css'
            );
        }
    }
    
    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path . '/views/js/front.js');
        $this->context->controller->addCSS(
            $this->_path . '/views/css/front.css'
        );
    }
    
    private function installXmlModules()
    {
        $obj = \simplexml_load_file($this->tab_modules_list);
        if (!$obj) {
            return false;
        }
        foreach ($obj->tab as $XMLElement) {
            foreach ($XMLElement->attributes() as $key => $value) {
                $class_value = (array)$value;
                $moduleGroupName = isset($class_value[0]) ? $class_value[0] : '';
                if ($key === 'class_name' && $moduleGroupName === 'AdminPayment') {
                    $modules = $XMLElement->xpath('module');
                    foreach ($modules as $moduleValue) {
                        foreach ($moduleValue->attributes() as $moduleAttrKey => $moduleAttrValue) {
                            $moduleAttrValue = (array)$moduleAttrValue;
                            $moduleValueName = isset($moduleAttrValue[0]) ? $moduleAttrValue[0] : '';
                            if ($moduleAttrKey === 'name' && $moduleValueName === self::MODULE_NAME) {
                                break 3;
                            }
                        }
                    }
                    $count = count($modules) + 1;
                    $module = $XMLElement->addChild('module');
                    $module->addAttribute('name', self::MODULE_NAME);
                    $module->addAttribute('position', $count);
                    break 2;
                }
            }
        }
        $obj->saveXML($this->tab_modules_list);
        return true;
    }
    
    private function installSql()
    {
        $sql_list = array(
            'CREATE TABLE IF NOT EXISTS `' . _DB_PREFIX_ . 'yabandpay_log` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT,
  `order_id` varchar(64) NOT NULL,
  `trade_id` varchar(64) NOT NULL,
  `transcation_id` varchar(64) DEFAULT NULL,
  PRIMARY KEY (`id`),
  UNIQUE KEY `trade_id` (`trade_id`) USING HASH
) ENGINE=' . _MYSQL_ENGINE_ . ';'
        );
        foreach ($sql_list as $sql) {
            if (!Db::getInstance()->execute($sql)) {
                return false;
            }
        }
        return true;
    }
    
    private function installOrderState()
    {
        foreach (\array_keys($this->pay_status) as $pay_status) {
            if (!Configuration::get($pay_status)
                || !Validate::isLoadedObject(
                    new OrderState(
                        Configuration::get(
                            $pay_status
                        )
                    )
                )
            ) {
                $order_state = new OrderState();
                $order_state->name = array();
                foreach (Language::getLanguages() as $language) {
                    if (Tools::strtolower($language['iso_code']) == 'zh') {
                        $order_state->name[$language['id_lang']]
                            = $this->pay_status[$pay_status]['zh'];
                    } else {
                        $order_state->name[$language['id_lang']]
                            = $this->pay_status[$pay_status]['en'];
                    }
                }
                $order_state->color = '#4169E1';
                $order_state->hidden = false;
                $order_state->delivery = false;
                $order_state->logable = false;
                $order_state->invoice = false;
                if ($order_state->add()) {
                    $source = dirname(__FILE__) . '/logo.png';
                    $destination = _PS_ROOT_DIR_ . '/img/os/'
                        . (int)$order_state->id . '.gif';
                    \copy($source, $destination);
                }
                Configuration::updateValue($pay_status, (int)$order_state->id);
            }
        }
        
        return true;
    }
    
    private function registrationHook()
    {
        if (!$this->registerHook('paymentOptions')
            || !$this->registerHook('paymentReturn')
            || !$this->registerHook('displayOrderConfirmation')
            || !$this->registerHook('displayAdminOrder')
            || !$this->registerHook('actionOrderStatusPostUpdate')
            || !$this->registerHook('actionOrderStatusUpdate')
            || !$this->registerHook('header')
            || !$this->registerHook('actionObjectCurrencyAddAfter')
            || !$this->registerHook('displayBackOfficeHeader')
            || !$this->registerHook('displayFooterProduct')
            || !$this->registerHook('actionBeforeCartUpdateQty')
            || !$this->registerHook('displayReassurance')
            || !$this->registerHook('displayInvoiceLegalFreeText')
            || !$this->registerHook('actionAdminControllerSetMedia')
            || !$this->registerHook('displayMyAccountBlock')
            || !$this->registerHook('displayCustomerAccount')
            || !$this->registerHook('displayShoppingCartFooter')
            || !$this->registerHook('actionCartSave')
        ) {
            return false;
        }
        return true;
    }
    
    private function uninstallSQL()
    {
        $sql_list = array(
            'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . 'yabandpay_log`',
            'DROP TABLE IF EXISTS `' . _DB_PREFIX_ . '_yabandpay_log`'
        );
        foreach ($sql_list as $sql) {
            if (!DB::getInstance()->execute($sql)) {
                return false;
            }
        }
        return true;
    }
    
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return false;
        }
        $active = Configuration::get('YABANDPAY_ACTIVE');
        if (empty($active) || !$active) {
            return false;
        }
        $options = array();
        //$description = Configuration::get('YABANDPAY_DESC') ?: 'Better Exchange Rate,  More Savings';
        $wechatpay = Configuration::get('YABANDPAY_WECHATPAY');
        if ($wechatpay) {
            /**
             * @var LanguageCore $language
             */
            $language = $this->context->language;
            if ($language->language_code === 'zh-cn'
                || $language->language_code === 'zh-tw'
            ) {
                $wechatpay_logo = Media::getMediaPath(
                    _PS_MODULE_DIR_ . '/'
                    . $this->name
                    . '/views/img/yabandpay_wechatpay_zh_100_32.png'
                );
            } else {
                $wechatpay_logo = Media::getMediaPath(
                    _PS_MODULE_DIR_ . '/'
                    . $this->name
                    . '/views/img/yabandpay_wechatpay_en_100_32.png'
                );
            }
            $wechatpay = new PaymentOption();
            $wechatpay->setModuleName($this->name)
                ->setLogo($wechatpay_logo)
                ->setCallToActionText(
                    $this->l(
                        'WeChat Pay'
                    )
                )
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'wechatpay'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . $this->l('Better Exchange Rate,  More Savings')
                    . '</p></section>'
                );
            $options[] = $wechatpay;
        }

        $alipay = Configuration::get('YABANDPAY_ALIPAY');
        if ($alipay) {
            $alipay = new PaymentOption();
            $alipay->setModuleName($this->name)
                ->setLogo(
                    Media::getMediaPath(
                        _PS_MODULE_DIR_ . '/'
                        . $this->name
                        . '/views/img/yabandpay_alipay_en_100_32.png'
                    )
                )
                ->setCallToActionText($this->l('Alipay'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'alipay'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . $this->l('Better Exchange Rate,  More Savings')
                    . '</p></section>'
                );
            $options[] = $alipay;
        }

        $ideal = Configuration::get('YABANDPAY_IDEAL');
        if ($ideal) {
            $ideal = new PaymentOption();
            $ideal->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_ideal_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('iDeal'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'ideal'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用iDeal'
                    . '</p></section>'
                );
            $options[] = $ideal;
        }

        $sofort = Configuration::get('YABANDPAY_SOFORT');
        if ($sofort) {
            $sofort = new PaymentOption();
            $sofort->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_sofort_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('Sofort'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'sofort'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用Sofort'
                    . '</p></section>'
                );
            $options[] = $sofort;
        }

        $bancontact = Configuration::get('YABANDPAY_BANCONTACT');
        if ($bancontact) {
            $bancontact = new PaymentOption();
            $bancontact->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_bancontact_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('Bancontact'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'bancontact'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用Bancontact'
                    . '</p></section>'
                );
            $options[] = $bancontact;
        }

        $visa = Configuration::get('YABANDPAY_VISA');
        if ($visa) {
            $visa = new PaymentOption();
            $visa->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_visa_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('Visa'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'visa'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用Visa'
                    . '</p></section>'
                );
            $options[] = $visa;
        }

        $mastercard = Configuration::get('YABANDPAY_MASTERCARD');
        if ($mastercard) {
            $mastercard = new PaymentOption();
            $mastercard->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_mastercard_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('Mastercard'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'mastercard'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用Mastercard'
                    . '</p></section>'
                );
            $options[] = $mastercard;
        }

        $paypal = Configuration::get('YABANDPAY_PAYPAL');
        if ($paypal) {
            $paypal = new PaymentOption();
            $paypal->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_paypal_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('PayPal'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'paypal'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用PayPal'
                    . '</p></section>'
                );
            $options[] = $paypal;
        }

        // $unionpay = Configuration::get('YABANDPAY_UNIONPAY');
        // if ($unionpay) {
        //     $unionpay = new PaymentOption();
        //     $unionpay->setModuleName($this->name)
        //         // ->setLogo(
        //         //     Media::getMediaPath(
        //         //         _PS_MODULE_DIR_ . '/'
        //         //         . $this->name
        //         //         . '/views/img/yabandpay_unionpay_en_100_32.png'
        //         //     )
        //         // )
        //         ->setCallToActionText($this->l('UnionPay'))
        //         ->setAction(
        //             $this->context->link->getModuleLink(
        //                 $this->name,
        //                 'payment',
        //                 array(
        //                     'type' => 'unionpay'
        //                 ),
        //                 true
        //             )
        //         )
        //         ->setAdditionalInformation(
        //             '<section><p>'
        //             . '欢迎使用UnionPay'
        //             . '</p></section>'
        //         );
        //     $options[] = $unionpay;
        // }

        $unionpaywap = Configuration::get('YABANDPAY_UNIONPAYWAP');
        if ($unionpaywap) {
            $unionpaywap = new PaymentOption();
            $unionpaywap->setModuleName($this->name)
                // ->setLogo(
                //     Media::getMediaPath(
                //         _PS_MODULE_DIR_ . '/'
                //         . $this->name
                //         . '/views/img/yabandpay_unionpay_en_100_32.png'
                //     )
                // )
                ->setCallToActionText($this->l('UnionPayWap'))
                ->setAction(
                    $this->context->link->getModuleLink(
                        $this->name,
                        'payment',
                        array(
                            'type' => 'unionpaywap'
                        ),
                        true
                    )
                )
                ->setAdditionalInformation(
                    '<section><p>'
                    . '欢迎使用UnionPayWap'
                    . '</p></section>'
                );
            $options[] = $unionpaywap;
        }

        return $options;
    }
    
    public function hookDisplayOrderConfirmation(array $params)
    {
        if (!$this->active) {
            return;
        }
        /**
         * @var Order $order
         */
        $order = $params['order'];
        $state = $order->getCurrentState();
        if (\in_array(
            $state,
            array(
                Configuration::get('PS_OS_OUTOFSTOCK_UNPAID'),
                Configuration::get('YABANDPAY_OS_WAITING_WECHATPAY'),
                Configuration::get('YABANDPAY_OS_WAITING_ALIPAY'),
                Configuration::get('YABANDPAY_OS_WAITING_IDEAL'),
                Configuration::get('YABANDPAY_OS_WAITING_SOFORT'),
                Configuration::get('YABANDPAY_OS_WAITING_BANCONTACT'),
                Configuration::get('YABANDPAY_OS_WAITING_VISA'),
                Configuration::get('YABANDPAY_OS_WAITING_MASTERCARD'),
                Configuration::get('YABANDPAY_OS_WAITING_PAYPAL'),
                Configuration::get('YABANDPAY_OS_WAITING_UNIONPAY'),
            ),
            true
        )
        ) {
            try {
                $result = \str_replace('\\', '', Tools::getValue('resultData'));
                $response = \json_decode($result, true);
                if (\json_last_error()) {
                    throw new \InvalidArgumentException(
                        $this->l(
                            'Syntax error, malformed JSON.'
                        )
                    );
                }
                if (!isset($response['data']['trade_id'])
                    || empty($response['data']['trade_id'])
                ) {
                    throw new \InvalidArgumentException(
                        $this->l(
                            'Data error trade_id not found.'
                        )
                    );
                }
                $trade_id = $response['data']['trade_id'];
                $cryptography = new Cryptography($this->getYabandpayToken());
                if ($cryptography->verify($response) === false) {
                    throw new \InvalidArgumentException(
                        $this->l(
                            'sign verify error'
                        )
                    );
                } else {
                    $pay_info = $response['data'];
                }
                if (isset($pay_info['state'])
                    && !empty($pay_info['state'])
                    && $pay_info['state'] === Payment::PAY_PAID
                ) {
                    if (\in_array(
                        $state,
                        array(
                            Configuration::get('YABANDPAY_OS_WAITING_WECHATPAY'),
                            Configuration::get('YABANDPAY_OS_WAITING_ALIPAY'),
                            Configuration::get('YABANDPAY_OS_WAITING_IDEAL'),
                            Configuration::get('YABANDPAY_OS_WAITING_SOFORT'),
                            Configuration::get('YABANDPAY_OS_WAITING_BANCONTACT'),
                            Configuration::get('YABANDPAY_OS_WAITING_VISA'),
                            Configuration::get('YABANDPAY_OS_WAITING_MASTERCARD'),
                            Configuration::get('YABANDPAY_OS_WAITING_PAYPAL'),
                            Configuration::get('YABANDPAY_OS_WAITING_UNIONPAY'),
                        ),
                        true
                    )
                    ) {
                        $order->setCurrentState(
                            Configuration::get('PS_OS_PAYMENT')
                        );
                        Yabandpay::updateYabandpayLogTransactionIdByTradeId(
                            $pay_info['trade_id'],
                            $pay_info['transaction_id']
                        );
                    } elseif ($state === Configuration::get('PS_OS_OUTOFSTOCK_UNPAID')) {
                        $order->setCurrentState(
                            Configuration::get('PS_OS_OUTOFSTOCK_PAID')
                        );
                        Yabandpay::updateYabandpayLogTransactionIdByTradeId(
                            $pay_info['trade_id'],
                            $pay_info['transaction_id']
                        );
                    }
                }
            } catch (\Exception $e) {
                $error_msg = $e->getMessage();
                PrestaShopLogger::addLog($error_msg);
                return $this->displayError($this->l($error_msg));
            }
        }
    }
    
    public function hookActionCartSave(array $params)
    {
        $cookie = $this->context->cookie;

        if (!empty($cookie->id_cart)
            && !empty($cookie->yabandpay_cart_id)
            && !empty($cookie->yabandpay_trade_id)
            && $cookie->id_cart == $cookie->yabandpay_cart_id
        ){
            $api = new \YaBandPay\PersiLiao\Api(
                new \YaBandPay\PersiLiao\Account(
                    $this->getYabandPayAccount(),
                    $this->getYabandPayToken()
                ),
                new \YaBandPay\PersiLiao\Request(
                    new \YaBandPay\PersiLiao\Cryptography( $this->getYabandPayToken() )
                )
            );
            $result = $api->cancel($cookie->yabandpay_trade_id);
            unset($cookie->yabandpay_cart_id);
            unset($cookie->yabandpay_trade_id);
        }
    }

    private function postProcess()
    {
        if (Tools::isSubmit('btnSubmit')) {
            $account = Tools::getValue('yabandpay_account');
            $token = Tools::getValue('yabandpay_token');
            if (empty($account) || empty($token)) {
                return $this->displayError(
                    $this->l(
                        'Account or Token can not be empty.'
                    )
                );
            }
            Configuration::updateValue(
                'YABANDPAY_WECHATPAY',
                Tools::getValue('yabandpay_wechatpay_on')
            );
            Configuration::updateValue(
                'YABANDPAY_ALIPAY',
                Tools::getValue('yabandpay_alipay_on')
            );
            Configuration::updateValue(
                'YABANDPAY_IDEAL',
                Tools::getValue('yabandpay_ideal_on')
            );
            Configuration::updateValue(
                'YABANDPAY_SOFORT',
                Tools::getValue('yabandpay_sofort_on')
            );
            Configuration::updateValue(
                'YABANDPAY_BANCONTACT',
                Tools::getValue('yabandpay_bancontact_on')
            );
            Configuration::updateValue(
                'YABANDPAY_VISA',
                Tools::getValue('yabandpay_visa_on')
            );
            Configuration::updateValue(
                'YABANDPAY_MASTERCARD',
                Tools::getValue('yabandpay_mastercard_on')
            );
            Configuration::updateValue(
                'YABANDPAY_PAYPAL',
                Tools::getValue('yabandpay_paypal_on')
            );
            // Configuration::updateValue(
            //     'YABANDPAY_UNIONPAY',
            //     Tools::getValue('yabandpay_unionpay_on')
            // );
            Configuration::updateValue(
                'YABANDPAY_UNIONPAYWAP',
                Tools::getValue('yabandpay_unionpaywap_on')
            );
            Configuration::updateValue(
                'YABANDPAY_EXCHANGE_CURRENCY',
                \trim(
                    Tools::getValue(
                        'yabandpay_exchange_currency'
                    )
                )
            );
            Configuration::updateValue(
                'YABANDPAY_EXCHANGE_RATE',
                Tools::getValue('yabandpay_exchange_rate')
            );
            Configuration::updateValue(
                'YABANDPAY_EXCHANGE_FEE',
                Tools::getValue('yabandpay_exchange_fee')
            );
            
            Configuration::updateValue('YABANDPAY_ACCOUNT', $account);
            Configuration::updateValue('YABANDPAY_TOKEN', $token);
            try {
                Configuration::updateValue('YABANDPAY_ACTIVE', true);
                return $this->displayConfirmation(
                    $this->l(
                        'Configuration saved successfully.'
                    )
                );
            } catch (\Exception $e) {
                Configuration::updateValue('YABANDPAY_ACTIVE', false);
                return $this->displayError(
                    $this->l(
                        'Account or Token invalid.'
                    )
                );
            }
        }
        return '';
    }
    
    public function getContent()
    {
        $html = '';
        $html .= $this->postProcess();
        $html .= $this->renderForm();
        return $html;
    }
    
    public function renderForm()
    {
        $fields_form = array(
            'form' => array(
                'legend' => array(
                    'title' => $this->l('YaBand Pay Setting'),
                    'icon'  => 'icon-cogs'
                ),
                'input'  => array(
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l(Payment::WECHAT),
                        'name'    => 'yabandpay_wechatpay_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . Payment::WECHAT
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l(Payment::ALIPAY),
                        'name'    => 'yabandpay_alipay_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . Payment::ALIPAY
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l(Payment::IDEAL),
                        'name'    => 'yabandpay_ideal_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . Payment::IDEAL
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l('Sofort'),
                        'name'    => 'yabandpay_sofort_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . 'Sofort'
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l(Payment::BANCONTACT),
                        'name'    => 'yabandpay_bancontact_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . Payment::BANCONTACT
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l(Payment::VISA),
                        'name'    => 'yabandpay_visa_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . Payment::VISA
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l(Payment::MASTERCARD),
                        'name'    => 'yabandpay_mastercard_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . Payment::MASTERCARD
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l('PayPal'),
                        'name'    => 'yabandpay_paypal_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . 'PayPal'
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    // array(
                    //     'type'    => 'switch',
                    //     'label'   => $this->l('UnionPay'),
                    //     'name'    => 'yabandpay_unionpay_on',
                    //     'is_bool' => true,
                    //     'desc'    => $this->l(
                    //         'Enable/Disable ' . 'UnionPay'
                    //     ),
                    //     'values'  => array(
                    //         array(
                    //             'id'    => 'active_on',
                    //             'value' => true,
                    //             'label' => $this->l('Enabled')
                    //         ),
                    //         array(
                    //             'id'    => 'active_off',
                    //             'value' => false,
                    //             'label' => $this->l('Disabled')
                    //         )
                    //     ),
                    // ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l('UnionPayWap'),
                        'name'    => 'yabandpay_unionpaywap_on',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Enable/Disable ' . 'UnionPayWap'
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'active_on',
                                'value' => true,
                                'label' => $this->l('Enabled')
                            ),
                            array(
                                'id'    => 'active_off',
                                'value' => false,
                                'label' => $this->l('Disabled')
                            )
                        ),
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => 'YaBand Account',
                        'prefix'   => '<i class="icon icon-user"></i>',
                        'name'     => 'yabandpay_account',
                        'class'    => '" style="max-width:500px',
                        'required' => true
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => 'YaBand Key',
                        'name'     => 'yabandpay_token',
                        'prefix'   => '<i class="icon icon-key"></i>',
                        'required' => true,
                        'class'    => '" style="max-width:500px'
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Currency'),
                        'name'     => 'yabandpay_exchange_currency',
                        'desc'     => $this->l(
                            'You can set a currency which you want to use our payment, except for CNY and Euro.'
                        ),
                        'class'    => '" style="max-width:500px',
                        'required' => false
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Euro exchange rate'),
                        'name'     => 'yabandpay_exchange_rate',
                        'desc'     => $this->l(
                            'Please enter the Exchange Rate (1EUR=?)'
                        ),
                        'class'    => '" style="max-width:500px',
                        'required' => false
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Fee%'),
                        'name'     => 'yabandpay_exchange_fee',
                        //'desc' => $this->l('Please enter the fee'),
                        'class'    => '" style="max-width:500px',
                        'required' => false
                    )
                ),
                'submit' => array(
                    'title' => $this->l('Save'),
                )
            ),
        );
        
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->id = (int)Tools::getValue('id_carrier');
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'btnSubmit';
        $helper->currentIndex
            = $this->context->link->getAdminLink(
                'AdminModules',
                false
            )
            . '&configure=' . $this->name . '&tab_module=' . $this->tab
            . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFieldsValues(),
        );
        
        $this->fields_form = array();
        
        return $helper->generateForm(array($fields_form));
    }
    
    protected function getConfigFieldsValues()
    {
        return array(
            'yabandpay_account'           => Tools::getValue(
                'yabandpay_account',
                Configuration::get('YABANDPAY_ACCOUNT')
            ),
            'yabandpay_token'             => Tools::getValue(
                'yabandpay_token',
                Configuration::get('YABANDPAY_TOKEN')
            ),
            'yabandpay_wechatpay_on'      => Tools::getValue(
                'yabandpay_wechatpay_on',
                Configuration::get('YABANDPAY_WECHATPAY')
            ),
            'yabandpay_alipay_on'         => Tools::getValue(
                'yabandpay_alipay_on',
                Configuration::get('YABANDPAY_ALIPAY')
            ),
            'yabandpay_ideal_on'         => Tools::getValue(
                'yabandpay_ideal_on',
                Configuration::get('YABANDPAY_IDEAL')
            ),
            'yabandpay_sofort_on'         => Tools::getValue(
                'yabandpay_sofort_on',
                Configuration::get('YABANDPAY_SOFORT')
            ),
            'yabandpay_bancontact_on'         => Tools::getValue(
                'yabandpay_bancontact_on',
                Configuration::get('YABANDPAY_BANCONTACT')
            ),
            'yabandpay_visa_on'         => Tools::getValue(
                'yabandpay_visa_on',
                Configuration::get('YABANDPAY_VISA')
            ),
            'yabandpay_mastercard_on'         => Tools::getValue(
                'yabandpay_mastercard_on',
                Configuration::get('YABANDPAY_MASTERCARD')
            ),
            'yabandpay_paypal_on'         => Tools::getValue(
                'yabandpay_paypal_on',
                Configuration::get('YABANDPAY_PAYPAL')
            ),
            'yabandpay_unionpay_on'         => Tools::getValue(
                'yabandpay_unionpay_on',
                Configuration::get('YABANDPAY_UNIONPAY')
            ),
            'yabandpay_unionpaywap_on'         => Tools::getValue(
                'yabandpay_unionpaywap_on',
                Configuration::get('YABANDPAY_UNIONPAYWAP')
            ),
            'yabandpay_exchange_rate'     => Tools::getValue(
                'yabandpay_exchange_rate',
                Configuration::get('YABANDPAY_EXCHANGE_RATE')
            ),
            'yabandpay_exchange_fee'      => Tools::getValue(
                'yabandpay_exchange_fee',
                Configuration::get('YABANDPAY_EXCHANGE_FEE')
            ),
            'yabandpay_exchange_currency' => Tools::getValue(
                'yabandpay_exchange_currency',
                Configuration::get('YABANDPAY_EXCHANGE_CURRENCY')
            )
        );
    }
    
    public function getYabandPayAccount()
    {
        return Configuration::get('YABANDPAY_ACCOUNT');
    }
    
    public function getYabandPayToken()
    {
        return Configuration::get('YABANDPAY_TOKEN');
    }
    
    public function getUrl($controller, array $params = array())
    {
        return $this->context->link->getModuleLink(
            $this->name,
            $controller,
            $params,
            true
        );
    }
    
    public function getNotifyUrl()
    {
        return $this->getUrl('notify');
    }
    
    public function getSuccessUrl(Cart $cart)
    {
        return $this->context->link->getPageLink(
            'order-confirmation',
            true,
            (int)Context::getContext()->language->id,
            array(
                'id_cart'   => $cart->id,
                'id_module' => $this->id,
                'id_order'  => $this->currentOrder,
                'key'       => (new Customer($cart->id_customer))->secure_key
            )
        );
    }

    public function getRedirectUrl()
    {
        return $this->getUrl('redirect');
    }
    
    public static function addYabandpayLogTradeId($order_id, $trade_id)
    {
        $sql = 'INSERT INTO ' . _DB_PREFIX_
            . 'yabandpay_log (`order_id`,`trade_id`)VALUES(' . $order_id . ',\''
            . $trade_id . '\')';
        if (!DB::getInstance()->execute($sql)) {
            return false;
        }
        return true;
    }
    
    public static function updateYabandpayLogTransactionIdByTradeId(
        $trade_id,
        $transaction_id
    ) {
        $sql = 'UPDATE ' . _DB_PREFIX_ . 'yabandpay_log SET `transcation_id`=\''
            . $transaction_id . '\' WHERE `trade_id`=\'' . $trade_id . '\'';
        if (!DB::getInstance()->execute($sql)) {
            return false;
        }
        return true;
    }
}
